/*
Juan Antonio Infantes Díaz. (j.a.infantes.diaz@gmail.com)
OpenDomótica versión V1.0 (http://opendomotica.wordpress.com)
Fecha: 26/03/2009
Libería de interconexión de Arduino con el PC por puerto serie emulando X10.
*/

#include <EEPROM.h>
const int MEM = 3;
const char CASA = 'B';
int ledpin = 4;
int state[14];

char readCharacter(char &val)
{
  while(!Serial.available());
  val = char(Serial.read());
  return val;
}

int readInt(int &var)
{
  char tmp[2];
  char val;
  
  readCharacter(val);
  if ((val > '0') && (val <= '9')){ //Ejemplo 16
    tmp[0] = val;
    readCharacter(tmp[1]);
  }else{ // Ejmplo 01
     readCharacter(tmp[0]);
  }
  var = atoi(tmp);
  return var;
}

void setupEEPROM()
{
  int i;
  for(i=0;i<=512;i++)
    EEPROM.write(i,0);
}

void setupStates(){
  int i;
  for(i=0;i<=13;i++)
    state[i] = 0;
}

void setupPins(int entrada, int pin, int ES)
{
    //int i, n=3; //la variable n guarda las posiciones que hay que actualizar cada vez que se mete un código de entrada
    EEPROM.write(entrada * MEM, pin);
    EEPROM.write(entrada * MEM + 1, ES);
    EEPROM.write(entrada * MEM + 2, 1);    
    /*
    Primero se guarda la entrada correspondiente
    Segundo se guarda el tipo 0 = Entrada, 1 = Salida
    Tercero se guarda 0 = Inactivo, 1 = Activo  
    */
}

void getX10(char &HouseCode, int &KeyCode)
{
  readCharacter(HouseCode);
  readInt(KeyCode);  
}

void getPins(int &pin, int  &ES)
{
  readInt(pin);
  readInt(ES);
}

int readAction(int &param){
  char a [6];
  int i;
  for(i=1;i<6;i++) //inicializamos el array de caracteres a
    a[i] = 0;
  
  readCharacter(a[0]);
  if(strcmp(a,"+")==0){
    readInt(param);
    return 3;
  }else if(strcmp(a,"-")==0){
    readInt(param);
    return 4;
  }
  readCharacter(a[1]);
  if(strcmp(a,"on")==0){
      return 1;
  }
  readCharacter(a[2]);
  if(strcmp(a,"off")==0) {
      return 2;
  }
  readCharacter(a[3]);
  readCharacter(a[4]);
  if(strcmp(a,"state")==0){
      return 5;
  }else{
      return 0; 
  }
  
}
int getPin(int KeyCode){
  return EEPROM.read(KeyCode*MEM);  
}

void setup()
{
  Serial.begin(9600);
  //Inicializamos los pines de entrada y salida 
  int i;
  for(i=0;i<=13;i++)
    pinMode(i,OUTPUT);
  for(i=0;i<=512;i=i+3){
    if(EEPROM.read(i+2)){ //si está activo
      if(EEPROM.read(i+1)){ //es de salida
        pinMode(EEPROM.read(i),OUTPUT);
      }else{ //es de entrada
        pinMode(EEPROM.read(i),INPUT);        
      }
    }
  }
  pinMode(ledpin,OUTPUT); //este pin debe ser borrado, sólo indica que se ha grabado en la EEPROM
  setupStates();
}

void loop()
{
  char HouseCode;
  int KeyCode,pin,ES,accion,param;
  /*
  if(Serial.available()){
    Serial.println(readCharacter(HouseCode));
    Serial.println(readInt(pin));
    Serial.println(readInt(pin));
    Serial.println(readAction(pin));
  }*/
  

  getX10(HouseCode,KeyCode);
  
  Serial.println(HouseCode);
  Serial.println(KeyCode);
  
  
  
  switch(HouseCode){
    case 'Y':
      setupEEPROM();
      break;
    case 'X':
    getPins(pin,ES);
    setupPins(KeyCode,pin,ES);
    Serial.println(int(KeyCode));
    Serial.println(int(pin));
    Serial.println(int(ES));
    digitalWrite(ledpin,HIGH);
    delay(1000);
    digitalWrite(ledpin,LOW);
    HouseCode = 'Z';
    break;
    case CASA:
    accion = readAction(param);
    switch(accion){
    case 1: //on
      pin = getPin(KeyCode);
      digitalWrite(pin,HIGH);
      state[pin] = 255;
      break;
    case 2: //off
      pin = getPin(KeyCode);
      digitalWrite(pin,LOW);
      state[pin] = 0;
      break;
    case 3: //dim
      pin = getPin(KeyCode);
      state[pin] = param; 
      break;
    case 4: //bright
      pin = getPin(KeyCode);
      state[pin] = param; 
      break;
    case 5: //state
      pin = getPin(KeyCode);
      Serial.println(state[pin]);
      break;
    }
    HouseCode = 'Z';
    break;
  }/* */
}
